/*  zweic -- a compiler for Zwei
 *
 *  Stephane Micheloud & LAMP
 *
 *  $Id: Position.java 2152 2005-11-13 23:36:56 +0100 (Sun, 13 Nov 2005) michelou $
 */

package zweic;


/**
 * This class provides methods for encoding a location in
 * a source file (line/column) into a single integer.
 */
final class Position {

    /** number of bits reserved for the column.
     */
    private static final int columnBits = 12;

    private static final int columnMask = (1 << columnBits) - 1;

    /** undefined position
     */
    public static final int UNDEFINED = 0;

    /** first position in a source file
     */
    public static final int FIRST = (1 << columnBits) | 1;

    /**
     * Encode a line and column number into a single int
     */
    public static int encode(int line, int col) {
        return (line << columnBits) | (col & columnMask);
    }

    /**
     * Get the line number out of an encoded position
     */
    public static int line(int pos) {
        return pos >>> columnBits;
    }

    /**
     * Return the column number of an encoded position
     */
    public static int column(int pos) {
        return pos & columnMask;
    }


    public static String toString(int pos) {
        return "" + line(pos) + ":" + column(pos);
    }

}
